
#ifdef FEM_EP_EXPORTS
#define FEM_EP_API __declspec(dllexport) 
#else
#define FEM_EP_API __declspec(dllimport) 
#endif

namespace FiniteElementMethod
{
    //This class is exported from FEM_EP.DLL
    class FEM
    {
    public:

		//GID common header
		static FEM_EP_API struct GID_HEADER
		{
			char head[8];
			char dimension[10];
			char nD[2];
			char ElemType[9];
			char ElemName[12];
			char Nnode[6];
			int nN;
		};

		//common structure for nodes coordinates
		static FEM_EP_API struct NODE
		{			
			double x, y, z; //cartesian coordinates
			double r, t; //cyllindrical coordinates (with "z")
			int DoFx, DoFy, DoFz; //degrees of freedom
			double ux, uy, uz; //displacements
			double fx, fy, fz; //forces
		};
		
		static FEM_EP_API struct BOUNDARY
		{
			int *id_Fixed_Node;
			int n_Fixed_Nodes;
			int *id_Fixed_DoF;
			int n_Fixed_DoFs;
			int *id_Loaded_Node;
			int n_Loaded_Nodes;
			int *id_Loaded_DoF;
			int n_Loaded_DoFs;
		};
		
		//Updating displacements of nodes
		static FEM_EP_API void Update_Nodes( FEM::NODE *Nodes, int n_Nodes, double *U, double *F );

		//Matrix of elasticity
		static FEM_EP_API void SG_De( int nDim, double E, double v, double *De );

		//Mohr-Coulomb tangent matrix
		static FEM_EP_API void MC_SG_Dep( double *stress, int nDim, double phi, double psi, double coh, double *De, double *Dp, double *Dep );
		
		//Stress invariants (Smiths & Griffiths)
		static FEM_EP_API void SG_SigInv( double *stress, int nDim, double &sigma_m, double &ds_bar, double &theta_Lode );

		//Stress invariants (Potts & Zdravkovic)
		static FEM_EP_API void PZ_SigInv( double *stress, int nDim, double &p, double &J, double &theta_Lode );

		//Derivatives of plastic potential function (Smiths & Griffiths)
		static FEM_EP_API void MC_SG_Qder( double *stress, int nDim, double psi, double *dQds );

		//Yield function for a Mohr-Coulomb material (phi in degrees).
		static FEM_EP_API double MC_SG_F( double *stress, int nDim, double phi, double coh );
	
		//Computes phi (in degrees) from yield function for a Mohr-Coulomb material with given cohesion.
		static FEM_EP_API double MC_SG_F_Phi( double *stress, int nDim, double coh, double Phi_min, double Phi_max );

		//Cartesian to cylindric coordinates tensor transformation
		static FEM_EP_API void Cart2Cyl6( double x, double y, double *CartT6, double *CylT6 );
		
		//Cartesian to cylindric coordinates vector transformation
		static FEM_EP_API void Cart2Cyl3( double x, double y, double *CartV3, double *CylV3 );

		//Principal strains/stresses for 3D elements for specified Gauss point iG (return: max gamma/tau)
		static FEM_EP_API double Principal3D( double *x, double *p_x );
    };
	/* END of FEM class */

	/* TH10 : Tetrahedra 10-nodes element, 15-gauss points */
    //This class is exported from FEM_EP.DLL
	class TH10G15
	{
    public:
		
		//constants for TH10G15 element
		static FEM_EP_API const int nG = 15; //number of Gauss points: 15
		static FEM_EP_API const int nN = 10; //number of nodes: 10
		
		//Gauss points data
		static FEM_EP_API struct GAUSSPOINT
		{
			//automatically initializable constants
			double local_coordinates[3];//eta, ksi, dzeta
			double weight;
			double N[10]; //shape functions
			double dNdL[30]; //local derivatives of shape functions: 3x10

			//working variables
			double E, v, G, Phi, Psi, coh; //material parameters (kPa, radians)
			double dE[2]; //changing E - Runge-Kutta method
			double Jacobian; //determinant of Jacobi matrix
			double Dep[36], Dp[36], De[36], B[180]; //tangent and strain matrices
			double initial_stress[6], stress[6], dstress[6]; //total and increment of stress
			double initial_strain[6], strain[6], dstrain[6], dstrain_vp[6]; //total, increment and visco-plastic increment of strain
			double principal_strain[3], principal_stress[3]; //principal values of strains and stresses
			double max_gamma_strain; //maximum shape deformation
			double sig_m, ds_bar, Lode, F; //S-G invariants, yield criterion

			//reserved variables
			double reserved[3]; //in case of need
		};

		//Element data
		static FEM_EP_API struct ELEMENT
		{
			//Gauss points
			GAUSSPOINT Gauss[15];
			int id_Node[10], DoF[30];
			double Node_Coordinates[30]; //10x3
		};

		//initialization of Gauss points data
		static FEM_EP_API void CreateGaussPoints( GAUSSPOINT *GP );
		
		//Jacobi matrix + Jacobian and global derivatives of shape functions
		//B-matrix with global derivatives of shape functions for specified Gauss point
		static FEM_EP_API void make_GP_J_B( TH10G15::ELEMENT *Element, int n_Elements );
		
		//Stiffness of TH10 element for specified Gauss point iG
		static FEM_EP_API void make_K( TH10G15::ELEMENT *Element, int n_Elements, int n_Nodes, double *K, int tangent );

		//Integration of stresses over element
		static FEM_EP_API void make_Fint( TH10G15::ELEMENT *Element, int n_Elements, int n_Nodes, double *Finternal );

		//Updating Phi, yield criterion and tangent matrices in elements
		static FEM_EP_API void update_Phi_F_D( TH10G15::ELEMENT *Element, int n_Elements, double Phi_min, int AFR, double Psi );

		//Integration of elasto-plastic constitutive matrix by RK4
		static FEM_EP_API void dU_Eps_Sig_RK4( TH10G15::ELEMENT *Element, int n_Elements, double *dU, int n_Nodes, int tangent, double Phi_min, int AFR, double Psi );

		//Integration of elasto-plastic constitutive matrix by RK4
		static FEM_EP_API void dU_Eps_Sig_RK4( TH10G15::ELEMENT *Element, int n_Elements, double *dU, int n_Nodes );

		//Integration of elasto-plastic constitutive matrix by RK4 with "plasticity" forces
		static FEM_EP_API void dU_Eps_Sig_RK4( TH10G15::ELEMENT *Element, int n_Elements, double *dU, int n_Nodes, double *Fp );

		//Updating strains and stresses in elements
		static FEM_EP_API void update_Strain_Stress( TH10G15::ELEMENT *Element, int n_Elements );

		//renumbering nodes for each element for GID compatibility
		static FEM_EP_API int CA( int id );

		//Reading mesh (nodes and elements) generated by GID
		static FEM_EP_API int GID_mesh( char *Project_Name, FEM::NODE *Nodes, int &n_Nodes, TH10G15::ELEMENT *Elements, int &n_Elements );

		//Preparing GID results file
		static FEM_EP_API int save_GID( char *Project_Name, FEM::NODE *Nodes, int n_Nodes, TH10G15::ELEMENT *Elements, int n_Elements );

		//Saving results to GID file
		static FEM_EP_API int update_GID( char *Project_Name, TH10G15::ELEMENT *Elements, int n_Elements, int n_Nodes, double *U, double *F, int step );
				
		//Saving all data in txt file
		static FEM_EP_API void SaveTXT( TH10G15::ELEMENT *Element, int n_Elements );
	};
	/* END of TH10G15 class */
}
